"use strict";

var email = require("./email");
var parser = require("./parser/main");
var plugins = require("./plugins");
var settings = require("./settings");
var composer = require("./composer");
var setsCtrl = require("./sets/controller");
var adminPages = require("./adminPages");
var socketRoutes = require("./socketRoutes");

/*===================================================== Exports  =====================================================*/

exports.init = init;

exports.adminMenu = addNavigation;
exports.composerFormatting = addComposerButton;

exports.parse = parsePlain;
exports.parsePostData = parsePost;

exports.pluginDeactivation = plugins.onPluginDisabled;
exports.pluginActivation = plugins.onPluginEnabled;

exports.email = email.parse;

/*==================================================== Functions  ====================================================*/

function addNavigation(data, cb) {
  adminPages.addNavigation(data);
  cb(null, data);
}

function addComposerButton(data, cb) {
  composer.formatting(data.options);
  cb(null, data);
}

function parsePlain(text, cb) {
  cb(null, parser.parse(text));
}

function parsePost(data, cb) {
  //noinspection JSUnresolvedVariable
  data.postData.content = parser.parse(data.postData.content);
  cb(null, data);
}

function init(data, cb) {
  socketRoutes.init();
  settings.init.then(function () {
    return setsCtrl.resetActive();
  }).done(function () {
    parser.refresh().done();
  });
  adminPages.init(data, cb);
}
